<?php
class AntiBot {
    private $db;
    private $blacklistTable = 'blacklist';
    private $logTable = 'access_log';
    private $humanLogTable = 'human_log';
    private $mouseMovementsTable = 'mouse_movements';
    private $config;

    public function __construct($config, $dbFile = 'antibot.sqlite') {
        $this->config = $config;
        $this->db = new PDO('sqlite:' . $dbFile);
        $this->createTables();
    }

    private function createTables() {
        $this->db->exec("
            CREATE TABLE IF NOT EXISTS $this->blacklistTable (
                ip TEXT PRIMARY KEY,
                attempts INTEGER DEFAULT 0,
                last_attempt INTEGER
            );
        ");

        $this->db->exec("
            CREATE TABLE IF NOT EXISTS $this->logTable (
                ip TEXT,
                user_agent TEXT,
                timestamp INTEGER
            );
        ");

        $this->db->exec("
            CREATE TABLE IF NOT EXISTS $this->humanLogTable (
                ip TEXT,
                user_agent TEXT,
                timestamp INTEGER
            );
        ");

        $this->db->exec("
            CREATE TABLE IF NOT EXISTS $this->mouseMovementsTable (
                ip TEXT,
                data TEXT,
                timestamp INTEGER
            );
        ");
    }

    public function isBlacklisted($ip) {
        $stmt = $this->db->prepare("SELECT ip FROM $this->blacklistTable WHERE ip = :ip");
        $stmt->execute([':ip' => $ip]);
        return $stmt->fetch(PDO::FETCH_ASSOC) !== false;
    }

    public function blacklistIP($ip) {
        $stmt = $this->db->prepare("
            INSERT OR REPLACE INTO $this->blacklistTable (ip, attempts, last_attempt)
            VALUES (:ip, :attempts, :last_attempt)
        ");
        $stmt->execute([
            ':ip' => $ip,
            ':attempts' => 3,
            ':last_attempt' => time()
        ]);
    }

    public function logAccess($ip, $user_agent) {
        $stmt = $this->db->prepare("
            INSERT INTO $this->logTable (ip, user_agent, timestamp)
            VALUES (:ip, :user_agent, :timestamp)
        ");
        $stmt->execute([
            ':ip' => $ip,
            ':user_agent' => $user_agent,
            ':timestamp' => time()
        ]);
    }

    public function logHumanVisit($ip, $user_agent) {
        $stmt = $this->db->prepare("
            INSERT INTO $this->humanLogTable (ip, user_agent, timestamp)
            VALUES (:ip, :user_agent, :timestamp)
        ");
        $stmt->execute([
            ':ip' => $ip,
            ':user_agent' => $user_agent,
            ':timestamp' => time()
        ]);
    }

    public function checkBot($user_agent) {
        $botPatterns = ['bot', 'crawl', 'slurp', 'spider', 'curl', 'wget', 'python', 'scrapy'];
        foreach ($botPatterns as $pattern) {
            if (stripos($user_agent, $pattern) !== false) {
                return true;
            }
        }
        return false;
    }

    public function isUsingProxy($ip) {
        // Basic method to detect proxy using headers
        $proxy_headers = array(
            'HTTP_VIA',
            'HTTP_X_FORWARDED_FOR',
            'HTTP_FORWARDED_FOR',
            'HTTP_X_FORWARDED',
            'HTTP_FORWARDED',
            'HTTP_CLIENT_IP',
            'HTTP_FORWARDED_FOR_IP',
            'VIA',
            'X_FORWARDED_FOR',
            'FORWARDED_FOR',
            'X_FORWARDED',
            'FORWARDED',
            'CLIENT_IP',
            'FORWARDED_FOR_IP',
            'HTTP_PROXY_CONNECTION'
        );

        foreach ($proxy_headers as $header) {
            if (isset($_SERVER[$header])) {
                return true;
            }
        }
        return false;
    }

    public function isAllowedCountry($ip) {
        $acceptedCountries = explode(',', $this->config['country_code']);
        if (empty($acceptedCountries) || $this->config['country_code'] == '') {
            return true; // No country restrictions
        }
        // Use a geoip service to get the country code
        $details = json_decode(file_get_contents("http://www.geoplugin.net/json.gp?ip=" . $ip));
        if ($details && isset($details->geoplugin_countryCode)) {
            return in_array($details->geoplugin_countryCode, $acceptedCountries);
        } else {
            return false; // Could not determine country
        }
    }

    public function isRapidRequester($ip) {
        $timeWindow = time() - 60; // Last 60 seconds
        $stmt = $this->db->prepare("
            SELECT COUNT(*) as request_count FROM $this->logTable
            WHERE ip = :ip AND timestamp > :timeWindow
        ");
        $stmt->execute([
            ':ip' => $ip,
            ':timeWindow' => $timeWindow
        ]);
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        $requestCount = $row['request_count'] ?? 0;

        $threshold = 30; // Adjust as necessary
        return $requestCount > $threshold;
    }

    public function saveMouseData($ip, $mouseData) {
        $stmt = $this->db->prepare("
            INSERT INTO $this->mouseMovementsTable (ip, data, timestamp)
            VALUES (:ip, :data, :timestamp)
        ");
        $stmt->execute([
            ':ip' => $ip,
            ':data' => json_encode($mouseData),
            ':timestamp' => time()
        ]);
    }

    public function analyzeMouseData($ip) {
        $stmt = $this->db->prepare("
            SELECT data FROM $this->mouseMovementsTable WHERE ip = :ip ORDER BY timestamp DESC LIMIT 1
        ");
        $stmt->execute([':ip' => $ip]);
        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        if ($row) {
            $mouseData = json_decode($row['data'], true);
            // Simple analysis: Check if mouse data exists and has variability
            if (count($mouseData) < 10) {
                return true; // Likely a bot
            }
            // Additional analysis can be added here
        } else {
            return true; // No mouse data collected
        }
        return false; // Passed mouse movement check
    }

    public function recordCaptchaAttempt($ip, $success) {
        $stmt = $this->db->prepare("SELECT attempts FROM $this->blacklistTable WHERE ip = :ip");
        $stmt->execute([':ip' => $ip]);
        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        $attempts = $row['attempts'] ?? 0;

        if ($success) {
            $attempts = 0;
        } else {
            $attempts += 1;
        }

        if ($attempts >= 3) {
            $this->blacklistIP($ip);
        } else {
            $stmt = $this->db->prepare("
                INSERT OR REPLACE INTO $this->blacklistTable (ip, attempts, last_attempt)
                VALUES (:ip, :attempts, :last_attempt)
            ");
            $stmt->execute([
                ':ip' => $ip,
                ':attempts' => $attempts,
                ':last_attempt' => time()
            ]);
        }
    }

    public function processPostRequest() {
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $ip = $_SERVER['REMOTE_ADDR'];
            $user_agent = $_SERVER['HTTP_USER_AGENT'] ?? 'Unknown';

            $rawInput = file_get_contents('php://input');
            $data = json_decode($rawInput, true);

            if (isset($data['action']) && $data['action'] === 'save_mouse_data') {
                if (isset($data['mouseData'])) {
                    $this->saveMouseData($ip, $data['mouseData']);
                }
                exit; // Stop further processing
            }

            if (isset($_POST['action']) && $_POST['action'] === 'verify_captcha') {
                // Cloudflare Turnstile verification
                $secretKey = $this->config['secret_key']; // Use secret key from config
                $token = $_POST['cf-turnstile-response'];

                $data = [
                    'secret' => $secretKey,
                    'response' => $token,
                    'remoteip' => $ip
                ];

                $ch = curl_init('https://challenges.cloudflare.com/turnstile/v0/siteverify');
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));

                $response = curl_exec($ch);
                curl_close($ch);

                $responseData = json_decode($response);

                if ($responseData->success) {
                    $this->recordCaptchaAttempt($ip, true);
                    $this->logHumanVisit($ip, $user_agent);
                    $_SESSION['verified'] = time();
                    header('Location: ' . $_SERVER['REQUEST_URI']);
                    exit;
                } else {
                    $this->recordCaptchaAttempt($ip, false);
                    header('Location: ' . $this->config['finish']);
                    exit;
                }
            }
        }
    }
}
?>
